# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-10-12 19:14
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('councilmatic_core', '0012_auto_20161012_1218'),
    ]

    operations = [
        migrations.AlterField(
            model_name='organization',
            name='source_url',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]
