# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-10-14 18:49
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('councilmatic_core', '0020_auto_20161013_1704'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='billdocument',
            name='document',
        ),
        migrations.RemoveField(
            model_name='eventdocument',
            name='document',
        ),
        migrations.RemoveField(
            model_name='eventdocument',
            name='updated_at',
        ),
        migrations.AddField(
            model_name='billdocument',
            name='full_text',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='billdocument',
            name='note',
            field=models.TextField(default=''),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='billdocument',
            name='url',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='eventdocument',
            name='full_text',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='eventdocument',
            name='note',
            field=models.TextField(default=''),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='eventdocument',
            name='url',
            field=models.TextField(blank=True),
        ),
        migrations.DeleteModel(
            name='Document',
        ),
    ]
