<div class='well'>
    <h4>
    	<i class='fa fa-fw fa-info-circle'></i> What does City Council do?
    </h4>
    <p>
    	{{CITY_COUNCIL_NAME}} monitors the operation and performance of city agencies, makes land use decisions and has sole responsibility for approving the city's budget. It also legislates on a wide range of other subjects. The Council is an equal partner with the Mayor in the governing of {{ CITY_NAME }}.
    </p>
    <p>
    	<a href='/about/#about-city-council'>Read more <i class='fa fa-chevron-right'></i></a>
    </p>
</div>